<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class provides User Account Update functions
 *
 * @package    lnApp
 * @category   Controllers/User
 * @author     Deon George
 * @copyright  (c) 2009-2013 Deon George
 * @license    http://dev.leenooks.net/license.html
 */
abstract class lnApp_Controller_User_Account extends Controller_Account {
	protected $auth_required = TRUE;

	protected $secure_actions = array(
		'edit'=>0,
		'resetpassword'=>0,
	);

	/**
	 * Enable User to Edit their Account Details
	 */
	public function action_edit() {
		if ($_POST AND $this->ao->values($_POST)->changed() AND (! $this->save($this->ao)))
			$this->ao->reload();

		Block::factory()
			->title(sprintf('Account: %s',$this->ao->accnum()))
			->title_icon('fa-wrench')
			->type('form-horizontal')
			->body(View::factory('account/user/edit')->set('o',$this->ao));
	}

	public function action_resetpassword() {
		if ($this->request->post()) {
			$validation = Validation::factory($this->request->post())
				->rule('password','not_empty')
				->rule('password','min_length',array(':value',6))
				->rule('password_confirm','matches',array(':validation',':field','password'));

			// Store our new values
			$this->ao->values($this->request->post());

			if (! $validation->check())
				SystemMessage::factory()
					->title(_('Record NOT updated'))
					->type('error')
					->body(_('Your password didnt pass validation.'));

			// Run validation and save
			elseif ($this->ao->changed())
				if ($this->ao->save()) {
					SystemMessage::factory()
						->title('Record updated')
						->type('success')
						->body(_('Your account record has been updated.'));

					// Log the password reset
					$this->ao->log('Password reset');

				}

			HTTP::redirect('login');
		}

		Block::factory()
			->title(sprintf('Password Reset: %s',$this->ao->accnum()))
			->title_icon('fa-cog')
			->id('reset')
			->type('form-horizontal')
			->body(View::factory('account/user/resetpassword')->set('o',$this->ao));
	}
}
?>
